
import java.security.*;
import java.lang.reflect.*;
import java.util.*;
import java.io.*;
import org.apache.tools.ant.*;
import org.apache.tools.ant.types.*;
import org.apache.tools.ant.taskdefs.*;
import org.apache.tools.ant.util.*;

public class RightAnt extends Task {
    private static class ExitPreventor extends SecurityException {
        public ExitPreventor() {
        }
    }
    
    private Commandline cmdl = new Commandline();
    public Commandline.Argument createArg() {
        return cmdl.createArgument();
    }
    
    public void execute() throws BuildException {
        final String[] args = cmdl.getArguments();
        final Throwable[] exception = { null };
        final Permission setSecurityManagerPermission = new RuntimePermission("setSecurityManager");
        final Permission exitVMPermission = new RuntimePermission("exitVM");
        final boolean[] finished = { false };
        final Integer[] firstStatus = new Integer[1];
        
        final SecurityManager newSecurityManager = new SecurityManager() {
            public synchronized void checkPermission(Permission perm) {
                if (!finished[0] && perm.implies(setSecurityManagerPermission))
                    throw new SecurityException();
                if (perm.implies(exitVMPermission))
                    throw new ExitPreventor();
            }
            public synchronized void checkExit(int status) {
                if (firstStatus[0] == null)
                    firstStatus[0] = new Integer(status);
                throw new ExitPreventor();
            }
        };
        
        synchronized (System.class) {
            SecurityManager oldSecurityManager = System.getSecurityManager();
            if (oldSecurityManager != null)
                throw new BuildException("SecurityManager exists");
            
            System.setSecurityManager(newSecurityManager);
        }
        
        try {
            Thread t = new Thread(new ThreadGroup("rightant"), new Runnable() {
                public void run() {
                    try {
                        ClassLoader loader = new ClassLoader() { };
                        Class clazz = loader.loadClass("org.apache.tools.ant.Main");
                        Method method = clazz.getMethod("main", new Class[] { String[].class });
                        method.invoke(null, new Object[] { args });
                    } catch (Throwable x) {
                        if (x instanceof InvocationTargetException)
                            x = ((InvocationTargetException) x).getCause();
                        exception[0] = x;
                    }
                }
            });
            System.out.println("*** start of sub-ant");
            t.start();
            for (;;) {
                try {
                    t.join();
                    break;
                } catch (InterruptedException x) {
                }
            }
            System.out.println("*** end of sub-ant");
            if (exception[0] != null) {
                Throwable x = exception[0];
                if (x instanceof ExitPreventor) {
                    ExitPreventor preventor = (ExitPreventor) x;
                    if (firstStatus[0] == null || firstStatus[0].intValue() == 0)
                        return;
                    throw new BuildException();
                }
                if (x instanceof Error)
                    throw (Error) x;
                if (x instanceof BuildException)
                    throw (BuildException) x;
                throw new BuildException(x);
            }
        } finally {
            finished[0] = true;
            synchronized (System.class) {
                System.setSecurityManager(null);
            }
        }
    }
}



